local plugin = RW:GetPlugin("Options")
if not plugin then return end

local AceGUI = LibStub("AceGUI-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local party
local partyNames = {}
local guildNames = {}
local selectedSortMethod
local displayTree
local updateSchedule
local listFontSize = 15

local sortmethods = {
	["name"] = "By Name",
	["version"] = "By Version",
}

local function sortByName(a, b)
	return a.name < b.name
end

local function sortByVersion(a, b)
	local aVersion = a.version or 999
	local bVersion = b.version or 999
	if (a.version and b.version) and a.version == b.version then
		if a.revision == b.revision then
			if a.releaseType == b.releaseType then
				return a.name < b.name
			else
				return a.releaseType < b.releaseType
			end
		else
			return a.revision < b.revision
		end
	else
		return a.name < b.name
	end
end

local function selectSortMethod(widget, event, method)	
	plugin.db.profile.VersionSort = method
	plugin:GetVersionDisplay(displayTree)
end

selectedSortMethod = sortByName

local function updateParty()
	party = party or RW:GetPlugin("Party")
	wipe(partyNames)
	if party:GetPartyType() ~= "none" then
		
		for name, data in party:PartyIterator() do
			tinsert(partyNames, data)
		end
		table.sort(partyNames, selectedSortMethod)
	end
end

local function updateGuild()
	wipe(guildNames)
	if RW.GuildVersions then
		for name, data in pairs(RW.GuildVersions) do
			tinsert(guildNames, data)
		end
		table.sort(guildNames, selectedSortMethod)
	end
end

local function addNames(listType, list, group)
	local sgroup = AceGUI:Create("OptionFrame")
	sgroup:SetTitle(listType)
	sgroup:SetLayout("Flow")
	sgroup:SetFullWidth(true)
	
	local col = AceGUI:Create("Label")
	col:SetFont(LSM:Fetch("font", "accid"), listFontSize, "OUTLINE")
	col:SetText("Name")
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	col = AceGUI:Create("Label")
	col:SetFont(LSM:Fetch("font", "accid"), listFontSize, "OUTLINE")
	col:SetText("Version")
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	col = AceGUI:Create("Label")
	col:SetFont(LSM:Fetch("font", "accid"), listFontSize, "OUTLINE")
	col:SetText("Revisions")
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	col = AceGUI:Create("Label")
	col:SetFont(LSM:Fetch("font", "accid"), listFontSize, "OUTLINE")
	col:SetText("Type")
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	local sep = AceGUI:Create("Label")
	sep:SetText(" ")
	sep:SetFullWidth(true)
	sgroup:AddChild(sep)
	
	local color1 = {1, 0.6, 0.16}
	local color2 = {1, 1, 1}
	
	for i, data in pairs(list) do
		local color
		if i % 2 == 0 then
			color = color1
		else
			color = color2
		end
	
		col = AceGUI:Create("Label")
		col:SetFont(LSM:Fetch("font", "accid"), listFontSize, "OUTLINE")
		col:SetText(data.name)
		col:SetColor(unpack(color))
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
		
		col = AceGUI:Create("Label")
		col:SetFont(LSM:Fetch("font", "accid"), listFontSize, "OUTLINE")
		col:SetText(data.version or "---")
		col:SetColor(unpack(color))
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
		
		col = AceGUI:Create("Label")
		col:SetFont(LSM:Fetch("font", "accid"), listFontSize, "OUTLINE")
		col:SetText(data.revision or "---")
		col:SetColor(unpack(color))
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
		
		col = AceGUI:Create("Label")
		col:SetFont(LSM:Fetch("font", "accid"), listFontSize, "OUTLINE")
		col:SetText(data.releaseType or "---")
		col:SetColor(unpack(color))
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
	end
	
	group:AddChild(sgroup)
end

function plugin:GetVersionDisplay(tree)
	displayTree = tree
	tree:ReleaseChildren()
		
	local group = AceGUI:Create("ScrollFrame")
	group:SetLayout("Flow")
		
	local titel = AceGUI:Create("Label")
	titel:SetFont(LSM:Fetch("font", "accid"), 25, "OUTLINE")
	titel:SetText("Versions")
	titel:SetFullWidth(true)	
	group:AddChild(titel)
	
	local sortSelect = AceGUI:Create("Dropdown")
	sortSelect:SetCallback("OnValueChanged", selectSortMethod)
	sortSelect:SetList(sortmethods)
	sortSelect:SetValue(self.db.profile.VersionSort)
	sortSelect:SetLabel("Sorting")
	group:AddChild(sortSelect)
	
	local refreshButton = AceGUI:Create("Button")
	refreshButton:SetText("Refresh")
	refreshButton:SetCallback("OnClick", function()
		plugin:GetVersionDisplay(displayTree)
	end)
	group:AddChild(refreshButton)
	
	if self.db.profile.VersionSort == "name" then
		selectedSortMethod = sortByName
	elseif self.db.profile.VersionSort == "version" then
		selectedSortMethod = sortByVersion
	end
	
	updateParty()
	if #partyNames > 0 then
		addNames("Party", partyNames, group)		
	end
	
	updateGuild()
	if #guildNames > 0 then
		addNames("Guild", guildNames, group)	
	end

	tree:AddChild(group)
end